# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flatbuf

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

# A set of grouping keys
class Grouping(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Grouping()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsGrouping(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # Grouping
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Expressions to group by
    # Grouping
    def Keys(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from vast_flatbuf.org.apache.arrow.computeir.flatbuf.Expression import Expression
            obj = Expression()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Grouping
    def KeysLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Grouping
    def KeysIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        return o == 0

def Start(builder): builder.StartObject(1)
def GroupingStart(builder):
    """This method is deprecated. Please switch to Start."""
    return Start(builder)
def AddKeys(builder, keys): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(keys), 0)
def GroupingAddKeys(builder, keys):
    """This method is deprecated. Please switch to AddKeys."""
    return AddKeys(builder, keys)
def StartKeysVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def GroupingStartKeysVector(builder, numElems):
    """This method is deprecated. Please switch to Start."""
    return StartKeysVector(builder, numElems)
def End(builder): return builder.EndObject()
def GroupingEnd(builder):
    """This method is deprecated. Please switch to End."""
    return End(builder)